@@ -20,7 +20,6 @@ translations.
 
 
 from __future__ import print_function, division, absolute_import
-from collections import namedtuple
 import math
 import logging
 
@@ -34,38 +33,48 @@ from .variant_sequence_in_reading_frame import (
 from .default_parameters import (
     MIN_TRANSCRIPT_PREFIX_LENGTH,
     MAX_REFERENCE_TRANSCRIPT_MISMATCHES,
+    INCLUDE_MISMATCHES_AFTER_VARIANT,
     PROTEIN_SEQUENCE_LENGTH,
     MIN_ALT_RNA_READS,
     MIN_VARIANT_SEQUENCE_COVERAGE,
     VARIANT_SEQUENCE_ASSEMBLY,
 )
 from .dataframe_builder import dataframe_from_generator
-
+from .value_object import ValueObject
 
 logger = logging.getLogger(__name__)
 
-# TranslationKey contains fields related to a translated protein sequence
-# which should be used to combine multiple equivalent mutated amino acid
-# sequences.
-TranslationKey = namedtuple("TranslationKey", (
-    # translated sequence of a variant sequence in the ORF established
-    # by a reference context
-    "amino_acids",
-    # half-open interval coordinates for variant amino acids
-    # in the translated sequence
-    "variant_aa_interval_start",
-    "variant_aa_interval_end",
-    # did the amino acid sequence end due to a stop codon or did we
-    # just run out of sequence context around the variant?
-    "ends_with_stop_codon",
-    # was the variant a frameshift relative to the reference sequence?
-    "frameshift"))
+class TranslationKey(ValueObject):
+    """
+    TranslationKey contains fields related to a translated protein sequence
+    which should be used to combine multiple equivalent mutated amino acid
+    sequences.
+    """
+    __slots__ = [
+        # translated sequence of a variant sequence in the ORF established
+        # by a reference context
+        "amino_acids",
+        # half-open interval coordinates for variant amino acids
+        # in the translated sequence
+        "variant_aa_interval_start",
+        "variant_aa_interval_end",
+        # did the amino acid sequence end due to a stop codon or did we
+        # just run out of sequence context around the variant?
+        "ends_with_stop_codon",
+        # was the variant a frameshift relative to the reference sequence?
+        "frameshift"
+    ]
 
-class Translation(object):
+class Translation(TranslationKey):
     """
     Translated amino acid sequence of a VariantSequenceInReadingFrame for a
     particular ReferenceContext and VariantSequence.
     """
+    __slots__ = [
+        "untrimmed_variant_sequence",
+        "reference_context",
+        "variant_sequence_in_reading_frame"
+    ]
 
     def __init__(
             self,
@@ -108,7 +117,9 @@ class Translation(object):
 
     @property
     def number_mismatches(self):
-        return self.variant_sequence_in_reading_frame.number_mismatches
+        """Only counting number of mismatches before the variant locus.
+        """
+        return self.variant_sequence_in_reading_frame.number_mismatches_before_variant
 
     @property
     def cdna_sequence(self):
@@ -143,6 +154,7 @@ class Translation(object):
             reference_context,
             min_transcript_prefix_length,
             max_transcript_mismatches,
+            include_mismatches_after_variant,
             protein_sequence_length=None):
         """
         Attempt to translate a single VariantSequence using the reading frame
@@ -164,6 +176,10 @@ class Translation(object):
             sequences disagrees at more than this number of positions before the
             variant nucleotides.
 
+        include_mismatches_after_variant : bool
+            If true, mismatches after the variant nucleotides will also count
+            against max_transcript_mismatches filtering.
+
         protein_sequence_length : int, optional
             Truncate protein to be at most this long
 
@@ -175,7 +191,8 @@ class Translation(object):
             variant_sequence,
             reference_context,
             min_transcript_prefix_length=min_transcript_prefix_length,
-            max_transcript_mismatches=max_transcript_mismatches)
+            max_transcript_mismatches=max_transcript_mismatches,
+            include_mismatches_after_variant=include_mismatches_after_variant)
 
         if variant_sequence_in_reading_frame is None:
             logger.info("Unable to determine reading frame for %s", variant_sequence)
@@ -328,6 +345,7 @@ def translation_generator(
         reference_contexts,
         min_transcript_prefix_length,
         max_transcript_mismatches,
+        include_mismatches_after_variant,
         protein_sequence_length=None):
     """
     Given all detected VariantSequence objects for a particular variant
@@ -353,6 +371,10 @@ def translation_generator(
         and reference transcript we're considering for determing the reading
         frame.
 
+    include_mismatches_after_variant : bool
+        If true, mismatches occurring after the variant locus will also count
+        toward max_transcript_mismatches filtering.
+
     protein_sequence_length : int, optional
         Truncate protein to be at most this long.
 
@@ -365,6 +387,7 @@ def translation_generator(
                 reference_context=reference_context,
                 min_transcript_prefix_length=min_transcript_prefix_length,
                 max_transcript_mismatches=max_transcript_mismatches,
+                include_mismatches_after_variant=include_mismatches_after_variant,
                 protein_sequence_length=protein_sequence_length)
             if translation is not None:
                 yield translation
@@ -378,6 +401,7 @@ def translate_variant_reads(
         min_variant_sequence_coverage=MIN_VARIANT_SEQUENCE_COVERAGE,
         min_transcript_prefix_length=MIN_TRANSCRIPT_PREFIX_LENGTH,
         max_transcript_mismatches=MAX_REFERENCE_TRANSCRIPT_MISMATCHES,
+        include_mismatches_after_variant=INCLUDE_MISMATCHES_AFTER_VARIANT,
         variant_sequence_assembly=VARIANT_SEQUENCE_ASSEMBLY):
     """
     Given a variant and its associated alt reads, construct variant sequences
@@ -418,6 +442,10 @@ def translate_variant_reads(
         Don't try to determine the reading frame for a transcript if more
         than this number of bases differ.
 
+    include_mismatches_after_variant : bool
+        Include mismatches after the variant locus in the count compared
+        against max_transcript_mismatches.
+
     variant_sequence_assembly : bool
         Use overlap assembly to construct longer variant cDNA sequences.
     """
@@ -460,6 +488,7 @@ def translate_variant_reads(
         reference_contexts=reference_contexts,
         min_transcript_prefix_length=min_transcript_prefix_length,
         max_transcript_mismatches=max_transcript_mismatches,
+        include_mismatches_after_variant=include_mismatches_after_variant,
         protein_sequence_length=protein_sequence_length))
 
 def translate_variants(
@@ -470,6 +499,7 @@ def translate_variants(
         min_variant_sequence_coverage=MIN_VARIANT_SEQUENCE_COVERAGE,
         min_transcript_prefix_length=MIN_TRANSCRIPT_PREFIX_LENGTH,
         max_transcript_mismatches=MAX_REFERENCE_TRANSCRIPT_MISMATCHES,
+        include_mismatches_after_variant=INCLUDE_MISMATCHES_AFTER_VARIANT,
         variant_sequence_assembly=VARIANT_SEQUENCE_ASSEMBLY):
     """
     Translates each coding variant in a collection to one or more protein
@@ -508,6 +538,10 @@ def translate_variants(
         Don't try to determine the reading frame for a transcript if more
         than this number of bases differ.
 
+    include_mismatches_after_variant : bool
+        Include mismatches after the variant locus in the count compared
+        against max_transcript_mismatches.
+
     variant_sequence_assembly : bool
         Use overlap assembly to construct longer variant cDNA sequences.
 
@@ -524,6 +558,7 @@ def translate_variants(
             min_variant_sequence_coverage=min_variant_sequence_coverage,
             min_transcript_prefix_length=min_transcript_prefix_length,
             max_transcript_mismatches=max_transcript_mismatches,
+            include_mismatches_after_variant=include_mismatches_after_variant,
             variant_sequence_assembly=variant_sequence_assembly)
         yield variant, translations
 
